// 013 u_c_path_segments
// 03 jan 2002

// -- (C) John COLIBRI 2006
// -- documentation: http://www.jcolibri.com

(*$r+*)

unit u_c_path_segments;
  interface
    uses Classes , u_c_basic_object;

    type c_path_segments= class(c_basic_object)
                            public
                              m_path: String;
                              m_c_segments: tStringList;

                              Constructor create_path_segments(p_name, p_path: String); Virtual;

                              function f_path_segment_count: Integer;
                              function f_path_segment(p_index: Integer): String;
                              procedure display_path_segments;
                              function f_absolute_path(p_relative_path: String): String;

                              Destructor Destroy; Override;
                          end;

    function f_common_path(pk_one, pk_two: String): String;
    function f_remove_middle_dot_dot(p_path: String): String;

  implementation
    uses u_c_display;

    // -- c_path_segments

    Constructor c_path_segments.create_path_segments(p_name, p_path: String);

      procedure build_segments;
        var l_index, l_start_index: Integer;
            l_segment_length: Integer;
            l_segment: String;
        begin
          if (Length(m_path)> 2) and (m_path[2]= ':') and (m_path[3]= '\')
            then begin
                m_c_segments.Add(m_path[1]+ ':');
                l_index:= 4;
              end
            else l_index:= 1;

          while l_index<= Length(m_path) do
          begin
            // -- sikp \
            while (l_index<= Length(m_path)) and (m_path[l_index]= '\') do
              Inc(l_index);

            // -- now get the segment
            l_start_index:= l_index;
            while (l_index<= Length(m_path)) and (m_path[l_index]<> '\') do
              Inc(l_index);

            l_segment_length:= l_index- l_start_index;
            if l_segment_length> 0
              then begin
                  SetLength(l_segment, l_segment_length);
                  Move(m_path[l_start_index], l_segment[1], l_segment_length);
                  m_c_segments.Add(l_segment);
                end
          end; // while l_index
        end; // build_segments

      begin // create_path_segments
        Inherited ;
        m_c_segments:= tStringList.Create;
        m_path:= p_path;

        build_segments;
      end; // create_path_segments

    function c_path_segments.f_path_segment_count: Integer;
      begin
        Result:= m_c_segments.Count;
      end; // f_path_segment_count

    function c_path_segments.f_path_segment(p_index: Integer): String;
      begin
        Result:= m_c_segments[p_index];
      end; // f_path_segment

    procedure c_path_segments.display_path_segments;
      var l_segment: Integer;
      begin
        for l_segment:= 0 to f_path_segment_count- 1 do
          display(f_path_segment(l_segment));
      end; // display_path_segments

    function c_path_segments.f_absolute_path(p_relative_path: String): String;
        // -- for project copier: .DOF contains now relative, and has to compute absolute
        // -- ex
        // -- c:\programs\colibri\utilities
        // --                              \exe
        // --                              \web\pascal_to_html
        // --   DOF: output is "..\..\exe
        // --   wants "c:\programs\colibri\utilities\exe"
      var l_length, l_index: Integer;
          l_level: Integer;
          l_relative_path: String;
      begin
        if Copy(p_relative_path, 1, 3)<> '..\'
          then Result:= p_relative_path
          else begin
              l_length:= Length(p_relative_path);
              // -- start at max level (project)
              l_level:= f_path_segment_count- 1;
              // -- back-up for each ..\ group
              l_index:= 1;
              l_relative_path:= p_relative_path;
              while (l_index+ 3<= l_length) and (copy(l_relative_path, 1, 3)= '..\') do
              begin
                Delete(l_relative_path, 1, 3);
                Dec(l_level);
                Inc(l_index, 3);
              end;

              // -- now build the path from the root
              Result:= '';
              for l_index:= 0 to l_level do
                Result:= Result+ f_path_segment(l_index)+ '\';
              // -- and add the final part
              Result:= Result+ l_relative_path;
            end;
      end; // f_absolute_path

    Destructor c_path_segments.Destroy;
      begin
        m_c_segments.Free;
        Inherited;
      end; // Destroy

    function f_common_path(pk_one, pk_two: String): String;
      var l_c_one, l_c_two: c_path_segments;
      var l_index: Integer;
      begin
        l_c_one:= c_path_segments.create_path_segments('', pk_one);
        l_c_two:= c_path_segments.create_path_segments('', pk_two);

        Result:= '';
        l_index:= 1;

        while (l_index< l_c_one.f_path_segment_count)
            and (l_index< l_c_two.f_path_segment_count)
            and (l_c_one.f_path_segment(l_index)= l_c_two.f_path_segment(l_index)) do
        begin
          result:= Result+ l_c_one.f_path_segment(l_index)+ '\';
          Inc(l_index);
        end;

        l_c_one.Free;
        l_c_two.Free;
      end; // f_common_path

    function f_remove_middle_dot_dot(p_path: String): String;
        // a\b\..\c    =>   a\c
      var l_segment: Integer;
          l_consecutive_dot_dot: Integer;
      begin
        with c_path_segments.create_path_segments('', p_path) do
        begin
          Result:= '';
          l_segment:= f_path_segment_count- 1;
          l_consecutive_dot_dot:= 0;

          while l_segment>= 0 do
          begin
            if f_path_segment(l_segment)= '..'
              then begin
                  Inc(l_consecutive_dot_dot);
                  Dec(l_segment);
                end
              else begin
                  // -- skip the backed up segments
                  while l_consecutive_dot_dot> 0 do
                  begin
                    Dec(l_consecutive_dot_dot);
                    Dec(l_segment);
                  end;

                  if l_segment>= 0
                    then Result:= f_path_segment(l_segment)+ '\'+ Result;

                  Dec(l_segment);
                end;
          end;

          Free;
        end;
      end; // f_remove_middle_dot_dot

    begin // u_c_path_segments
    end. // u_c_path_segments
